<?php

/**
 * Class Gateway
 */
class Sample_Gateway {
	
	protected static $instance;
	// Change gateway name before use 
	private $name = 'sample_gateway';
	
	/**
	 * @return Sample_Gateway
	 */
	public static function get_instance() {
		if (null === self::$instance) {
			self::$instance = new self();
		}
		
		return self::$instance;
	}
	
	/**
	 * Registers the your gateway
	 *
	 * @param $gateways
	 *
	 * @return mixed
	 */
	public function register_gateway($gateways) {
		$gateways[ $this->name ] = array('admin_label' => __('Sample Gateway', 'mprm-sample-gateway'), 'checkout_label' => __('Sample Gateway', 'mprm-sample-gateway'));
		
		return $gateways;
	}
	
	/**
	 * Render cc form if you need
	 */
	public function render_cc_form() {
		do_action('mprm_cc_form');
		/*
			<fieldset>
				<p>
					<label></label>
					<input type="text">
				</p>
			</fieldset>
		*/
	}
	
	/**
	 * Processes the payment
	 *
	 * @param $purchase_data
	 */
	public function payment_process($purchase_data) {
		
		global $mprm_options;
		
		/**********************************
		 * set transaction mode
		 **********************************/
		
		if (mprm_is_test_mode()) {
			$test_api_key = mprm_get_option('test_api_key');
			// set test credentials here
		} else {
			$live_api_key = mprm_get_option('live_api_key');
			// set live credentials here
		}
		
		/**********************************
		 * check for errors here
		 **********************************/
		
		/*
		// errors can be set like this
		if(!isset($_POST['card_number'])) {
			// error code followed by error message
			mprm_set_error('empty_card', __('You must enter a card number', 'mprm'));
		}
		*/
		
		// check for any stored errors
		$errors = mprm_get_errors();
		
		if (!$errors) {
			
			/**********************************
			 * setup the payment details
			 **********************************/
			/*
			$payment_data = array(
				'price' => total price of cart contents,
				'date' => date('Y-m-d H:i:s'),
				'user_email' => $user_email,
				'purchase_key' =>  // Random key,
				'currency' => mprm_get_currency(),
				'menu_items' => array of menu_item IDs,
				'user_info' => array of user's information
				'cart_details' => $purchase_data[ 'cart_details' ],
				'post_data' => $_POST,
				'status' => 'mprm-pending'
			);*/
			
			$payment_data = array(
				'price' => $purchase_data[ 'price' ],
				'date' => $purchase_data[ 'date' ],
				'user_email' => $purchase_data[ 'user_email' ],
				'purchase_key' => $purchase_data[ 'purchase_key' ],
				'currency' => mprm_get_currency(),
				'menu_items' => $purchase_data[ 'menu_items' ],
				'user_info' => $purchase_data[ 'user_info' ],
				'cart_details' => $purchase_data[ 'cart_details' ],
				'post_data' => $_POST,
				'status' => 'mprm-pending'
			);
			
			$merchant_payment_confirmed = true;
			
			// record the pending payment
			$payment_id = mprm_insert_payment($payment_data);
			
			/*
			if ( $redirect_to_merchant ){
				//redirect to merchant and verify payment with an IPN
				wp_redirect( ... );
				exit();
				//todo: webhooks/notifications/ipn listener, success and fail pages
			} else {
			
				//Process the credit card here
				$merchant_payment_confirmed = false;
				$response = wp_remote_post( ... );
				// if the merchant payment is complete, set a flag
			   
			     //todo: work with the response
			    if($response['status']){
					$merchant_payment_confirmed = true;
				}
			}
			*/
			
			if ($merchant_payment_confirmed) {
				
				// once a transaction is successful, set the purchase to complete
				mprm_update_payment_status($payment_id, 'publish');
				
				// go to the success page
				mprm_send_to_success_page('?payment_key=' . mprm_get_payment_key($payment_id));
				
			} else {
				//todo: note the error
				mprm_update_payment_status($payment_id, 'mprm-failed');
				mprm_insert_payment_note($payment_id, __('Payment failed due to invalid purchase key in PayPal IPN.', 'mp-restaurant-menu'));
				
			}
			
		} else {
			// if errors are present, send the user back to the purchase page so they can be corrected
			mprm_send_back_to_checkout('?payment-mode=' . $purchase_data[ 'post_data' ][ 'mprm-gateway' ]);
		}
	}
	
	/**
	 *
	 * @param $sections
	 *
	 * @return array
	 */
	public function add_settings_section($sections) {
		if (is_array($sections)) {
			$sections[ $this->name ] = __('Sample Gateway', 'mprm-sample-gateway');
		}
		
		return $sections;
	}
	
	/**
	 * Add settings
	 *
	 * @param $settings
	 *
	 * @return array
	 */
	public function add_settings($settings) {
		$sample_gateway_settings = array('sample_gateway' => array(
			array(
				'id' => 'sample_gateway_settings',
				'name' => '<strong>' . __('Sample Gateway Settings', 'mprm-sample-gateway') . '</strong>',
				'desc' => __('Configure the gateway settings', 'mprm-sample-gateway'),
				'type' => 'header'
			),
			array(
				'id' => 'live_api_key',
				'name' => __('Live API Key', 'mprm-sample-gateway'),
				'desc' => __('Enter your live API key', 'mprm-sample-gateway'),
				'type' => 'text',
				'size' => 'regular'
			),
			array(
				'id' => 'test_api_key',
				'name' => __('Test API Key', 'mprm-sample-gateway'),
				'desc' => __('Enter your test API key', 'mprm-sample-gateway'),
				'type' => 'text',
				'size' => 'regular'
			)
		));
		$settings = array_merge($settings, $sample_gateway_settings);
		
		return $settings;
	}
	
	/**
	 * Init payment action
	 */
	public function init_action() {
		add_action('mprm_payment_gateways', array($this, 'register_gateway'));
		add_action('mprm_gateway_' . $this->name, array($this, 'payment_process'));
		add_action('mprm_' . $this->name . '_cc_form', array($this, 'render_cc_form'));
		add_filter('mprm_settings_gateways', array($this, 'add_settings'));
		add_filter('mprm_settings_sections_gateways', array($this, 'add_settings_section'));
	}
}




